package com.yeejoin.amos.boot.module.hygf.biz.controller;

import org.springframework.web.bind.annotation.RequestMapping;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.Api;
import org.springframework.web.bind.annotation.RestController;
import com.yeejoin.amos.boot.biz.common.controller.BaseController;
import java.util.List;
import com.yeejoin.amos.boot.module.hygf.biz.service.impl.TrainingServiceImpl;
import org.typroject.tyboot.core.restful.utils.ResponseHelper;
import org.typroject.tyboot.core.restful.utils.ResponseModel;
import org.springframework.beans.factory.annotation.Autowired;
import javax.servlet.http.HttpServletRequest;
import org.springframework.web.bind.annotation.*;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yeejoin.amos.boot.module.hygf.api.dto.TrainingDto;
import org.typroject.tyboot.core.restful.doc.TycloudOperation;
import org.typroject.tyboot.core.foundation.enumeration.UserType;

/**
 * 培训信息表
 *
 * @author system_generator
 * @date 2024-03-11
 */
@RestController
@Api(tags = "培训信息表Api")
@RequestMapping(value = "/training")
public class TrainingController extends BaseController {

    @Autowired
    TrainingServiceImpl trainingServiceImpl;

    /**
     * 新增培训信息表
     *
     * @return
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
	@PostMapping(value = "/save")
	@ApiOperation(httpMethod = "POST", value = "新增培训信息表", notes = "新增培训信息表")
	public ResponseModel<TrainingDto> save(@RequestBody TrainingDto model) {
	    model = trainingServiceImpl.createWithModel(model);
	    return ResponseHelper.buildResponse(model);
	}

	/**
     * 根据sequenceNbr更新
     *
     * @param sequenceNbr 主键
     * @return
     */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@PutMapping(value = "/{sequenceNbr}")
	@ApiOperation(httpMethod = "PUT", value = "根据sequenceNbr更新培训信息表", notes = "根据sequenceNbr更新培训信息表")
	public ResponseModel<TrainingDto> updateBySequenceNbrTraining(@RequestBody TrainingDto model,@PathVariable(value = "sequenceNbr") Long sequenceNbr) {
	    model.setSequenceNbr(sequenceNbr);
	    return ResponseHelper.buildResponse(trainingServiceImpl.updateWithModel(model));
	}

	/**
     * 根据sequenceNbr删除
     *
     * @param sequenceNbr 主键
     * @return
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY)
    @DeleteMapping(value = "/{sequenceNbr}")
    @ApiOperation(httpMethod = "DELETE", value = "根据sequenceNbr删除培训信息表", notes = "根据sequenceNbr删除培训信息表")
    public ResponseModel<Boolean> deleteBySequenceNbr(HttpServletRequest request, @PathVariable(value = "sequenceNbr") Long sequenceNbr){
        return ResponseHelper.buildResponse(trainingServiceImpl.removeById(sequenceNbr));
    }

    /**
     * 根据sequenceNbr查询
     *
	 * @param sequenceNbr  主键
     * @return
     */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@GetMapping(value = "/{sequenceNbr}")
	@ApiOperation(httpMethod = "GET",value = "根据sequenceNbr查询单个培训信息表",  notes = "根据sequenceNbr查询单个培训信息表")
	public ResponseModel<TrainingDto> selectOne(@PathVariable Long sequenceNbr) {
	    return ResponseHelper.buildResponse(trainingServiceImpl.queryBySeq(sequenceNbr));
	}

	/**
	 * 列表全部数据查询
     *
	 * @return
	 */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation(httpMethod = "GET",value = "培训信息表列表全部数据查询",   notes = "培训信息表列表全部数据查询")
	@GetMapping(value = "/list")
	public ResponseModel<List<TrainingDto>> selectForList() {
	    return ResponseHelper.buildResponse(trainingServiceImpl.queryForTrainingList());
	}

	/**
	 * 列表分页查询
     *
	 * @return
	 */
	@TycloudOperation(ApiLevel = UserType.AGENCY)
	@ApiOperation(httpMethod = "GET",value = "培训信息表列表全部数据查询",   notes = "培训信息表列表全部数据查询")
	@GetMapping(value = "/page")
	public ResponseModel<Page<TrainingDto>> currentStudyStatusList(Integer current,
                                                                   Integer size,
                                                                   @RequestParam(required = false) String status,
                                                                   @RequestParam(required = false)  String name
                                                                   ) {
	    return ResponseHelper.buildResponse(trainingServiceImpl.currentStudyStatusList(current,size,   status,
                 name));
	}


}
